@@ -1,6 +1,6 @@
 # - coding: utf-8 -
 
-# Copyright (C) 2008 Toms Bauģis <toms.baugis at gmail.com>
+# Copyright (C) 2008-2009 Toms Bauģis <toms.baugis at gmail.com>
 
 # This file is part of Project Hamster.
 
@@ -26,88 +26,81 @@ import gtk
 import pango
 
 from hamster import dispatcher, storage, SHARED_DATA_DIR, stuff
-from hamster.charting import Chart
-from hamster.add_custom_fact import CustomFactController
+from hamster import charting
+
+from hamster.edit_activity import CustomFactController
 
 import datetime as dt
 import calendar
-import gobject
 import time
 
-class StatsViewer:
-    def __init__(self):
-        self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, "stats.glade"))
+class StatsViewer(object):
+    def __init__(self, parent = None):
+        self.parent = parent# determine if app shut shut down on close
+        self._gui = stuff.load_ui_file("stats.ui")
         self.window = self.get_widget('stats_window')
 
-        self.fact_tree = self.get_widget("facts")
-        self.fact_tree.set_headers_visible(False)
-        self.fact_tree.set_tooltip_column(1)
-        self.fact_tree.set_property("show-expanders", False)
+        #id, caption, duration, date (invisible), description, category
+        self.fact_store = gtk.TreeStore(int, str, str, str, str, str) 
+        self.setup_tree()
+        
+        graph_frame = self.get_widget("graph_frame")
+        background = (0.975,0.975,0.975)
+        graph_frame.modify_bg(gtk.STATE_NORMAL,
+                              gtk.gdk.Color(*[int(b*65536.0) for b in background]))
 
-        nameColumn = gtk.TreeViewColumn(_("Name"))
-        nameColumn.set_expand(True)
-        nameCell = gtk.CellRendererText()
-        nameCell.set_property("ellipsize", pango.ELLIPSIZE_END)
-        nameColumn.pack_start(nameCell, True)
-        nameColumn.set_cell_data_func(nameCell, self.parent_painter)
-        self.fact_tree.append_column(nameColumn)
+        
+        x_offset = 90 # let's nicely align all graphs
+        
+        self.category_chart = charting.BarChart(background = background,
+                                             bar_base_color = (238,221,221),
+                                             bars_beveled = False,
+                                             legend_width = x_offset,
+                                             max_bar_width = 35,
+                                             show_stack_labels = True
+                                             )
+        category_box = self.get_widget("totals_by_category")
+        category_box.add(self.category_chart)
+        category_box.set_size_request(130, -1)
+        
+
+        self.day_chart = charting.BarChart(background = background,
+                                        bar_base_color = (220, 220, 220),
+                                        bars_beveled = False,
+                                        show_scale = True,
+                                        max_bar_width = 35,
+                                        grid_stride = 4,
+                                        legend_width = 20)
+        self.get_widget("totals_by_day").add(self.day_chart)
+
+
+        self.activity_chart = charting.HorizontalBarChart(orient = "horizontal",
+                                             max_bar_width = 25,
+                                             values_on_bars = True,
+                                             stretch_grid = True,
+                                             legend_width = x_offset,
+                                             value_format = "%.1f",
+                                             background = background,
+                                             bars_beveled = False,
+                                             animate = False)
+        self.get_widget("totals_by_activity").add(self.activity_chart);
 
-        timeColumn = gtk.TreeViewColumn(_("Duration"))
-        timeCell = gtk.CellRendererText()
-        timeColumn.pack_end(timeCell, True)
-        timeColumn.set_cell_data_func(timeCell, self.duration_painter)
-        self.fact_tree.append_column(timeColumn)
-        
-        self.fact_store = gtk.TreeStore(int, str, str, str, str) #id, caption, duration, date (invisible), description
-        self.fact_tree.set_model(self.fact_store)
-        
-        x_offset = 80 # let's nicely align all graphs
-        
-        self.day_chart = Chart(max_bar_width = 40,
-                               collapse_whitespace = True,
-                               legend_width = x_offset)
-        eventBox = gtk.EventBox()
-        place = self.get_widget("totals_by_day")
-        eventBox.add(self.day_chart);
-        place.add(eventBox)
-        
-        self.category_chart = Chart(orient = "horizontal",
-                                    max_bar_width = 30,
-                                    animate=False,
-                                    values_on_bars = True,
-                                    stretch_grid = True,
-                                    legend_width = x_offset)
-        eventBox = gtk.EventBox()
-        place = self.get_widget("totals_by_category")
-        eventBox.add(self.category_chart);
-        place.add(eventBox)
-        
-        self.activity_chart = Chart(orient = "horizontal",
-                                    max_bar_width = 25,
-                                    animate = False,
-                                    values_on_bars = True,
-                                    stretch_grid = True,
-                                    legend_width = x_offset)
-        eventBox = gtk.EventBox()
-        place = self.get_widget("totals_by_activity")
-        eventBox.add(self.activity_chart);
-        place.add(eventBox)
         
         self.view_date = dt.date.today()
         
-        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1) #set to monday
+         #set to monday
+        self.start_date = self.view_date - \
+                                      dt.timedelta(self.view_date.weekday() + 1)
         # look if we need to start on sunday or monday
-        self.start_date = self.start_date + dt.timedelta(self.locale_first_weekday())
+        self.start_date = self.start_date + \
+                                      dt.timedelta(self.locale_first_weekday())
         
         self.end_date = self.start_date + dt.timedelta(6)
 
         
-        self.day_view = self.get_widget("day")
         self.week_view = self.get_widget("week")
         self.month_view = self.get_widget("month")
-
-        self.week_view.set_group(self.day_view)
-        self.month_view.set_group(self.day_view)
+        self.month_view.set_group(self.week_view)
         
         #initiate the form in the week view
         self.week_view.set_active(True)
@@ -117,12 +110,99 @@ class StatsViewer:
         dispatcher.add_handler('day_updated', self.after_fact_update)
 
         selection = self.fact_tree.get_selection()
-        selection.connect('changed', self.on_fact_selection_changed, self.fact_store)
+        selection.connect('changed', self.on_fact_selection_changed,
+                          self.fact_store)
+        self.popular_categories = [cat[0] for cat in storage.get_popular_categories()]
 
-        self.glade.signal_autoconnect(self)
+        self._gui.connect_signals(self)
         self.fact_tree.grab_focus()
+        
+        """
+        # this will help when profiling!
+        import gobject
+        self.i = 0
+        def redraw():
+            self.do_graph()
+            self.start_date -= dt.timedelta(7)
+            self.end_date -= dt.timedelta(7)
+            self.i +=1
+            return self.i < 50
+            
+        gobject.timeout_add(400, redraw)
+        """
         self.do_graph()
 
+    def more_on_left(self):
+        z = min(round((self.end_date - self.start_date).days / 21.0)+1, 5)
+        self.start_date = self.start_date - dt.timedelta(days = z)
+        self.do_graph()
+        
+    def less_on_left(self):
+        z = min(round((self.end_date - self.start_date).days / 21.0)+1, 5)
+        self.start_date = self.start_date + dt.timedelta(days=z)
+        self.do_graph()
+        
+    def setup_tree(self):
+        def parent_painter(column, cell, model, iter):
+            cell_text = model.get_value(iter, 1)
+            if model.iter_parent(iter) == None:
+                if model.get_path(iter) == (0,):
+                    text = '<span weight="heavy">%s</span>' % cell_text
+                else:
+                    text = '<span weight="heavy" rise="-20000">%s</span>' % cell_text
+                    
+                cell.set_property('markup', text)
+    
+            else:
+                activity_name = stuff.escape_pango(cell_text)
+                description = stuff.escape_pango(model.get_value(iter, 4))
+                category = stuff.escape_pango(model.get_value(iter, 5))
+
+                markup = stuff.format_activity(activity_name, category, description, pad_description = True)            
+                cell.set_property('markup', markup)
+
+        def duration_painter(column, cell, model, iter):
+            cell.set_property('xalign', 1)
+            cell.set_property('yalign', 0)
+    
+
+            text = model.get_value(iter, 2)
+            if model.iter_parent(iter) == None:
+                if model.get_path(iter) == (0,):
+                    text = '<span weight="heavy">%s</span>' % text
+                else:
+                    text = '<span weight="heavy" rise="-20000">%s</span>' % text
+            cell.set_property('markup', text)
+    
+
+        self.fact_tree = self.get_widget("facts")
+        self.fact_tree.set_headers_visible(False)
+        self.fact_tree.set_tooltip_column(1)
+        self.fact_tree.set_property("show-expanders", False)
+
+        # name
+        nameColumn = gtk.TreeViewColumn()
+        nameColumn.set_expand(True)
+        nameCell = gtk.CellRendererText()
+        nameCell.set_property("ellipsize", pango.ELLIPSIZE_END)
+        nameColumn.pack_start(nameCell, True)
+        nameColumn.set_cell_data_func(nameCell, parent_painter)
+        self.fact_tree.append_column(nameColumn)
+
+        # duration
+        timeColumn = gtk.TreeViewColumn()
+        timeCell = gtk.CellRendererText()
+        timeColumn.pack_end(timeCell, True)
+        timeColumn.set_cell_data_func(timeCell, duration_painter)
+
+
+
+
+        self.fact_tree.append_column(timeColumn)
+        
+        self.fact_tree.set_model(self.fact_store)
+        
+        
     def locale_first_weekday(self):
         """figure if week starts on monday or sunday"""
         import os
@@ -142,36 +222,7 @@ class StatsViewer:
             
         return first_weekday
         
-    def parent_painter(self, column, cell, model, iter):
-        cell_text = model.get_value(iter, 1)
-        if model.iter_parent(iter) == None:
-            if model.get_path(iter) == (0,):
-                text = '<span weight="heavy">%s</span>' % cell_text
-            else:
-                text = '<span weight="heavy" rise="-20000">%s</span>' % cell_text
-                
-            cell.set_property('markup', text)
-
-        else:
-            activity_name = stuff.escape_pango(cell_text)
-            description = stuff.escape_pango(model.get_value(iter, 4))
-    
-            text = "   %s" % activity_name
-            if description:
-                text+= """\n             <span style="italic" size="small">%s</span>""" % (description)
-                
-            cell.set_property('markup', text)
-
-    def duration_painter(self, column, cell, model, iter):
-        text = model.get_value(iter, 2)
-        if model.iter_parent(iter) == None:
-            if model.get_path(iter) == (0,):
-                text = '<span weight="heavy">%s</span>' % text
-            else:
-                text = '<span weight="heavy" rise="-20000">%s</span>' % text
-        cell.set_property('markup', text)
-
-    def get_facts(self):
+    def get_facts(self, facts):
         self.fact_store.clear()
         totals = {}
 
@@ -179,10 +230,6 @@ class StatsViewer:
         by_category = {}
         by_day = {}
 
-        week = {"days": [], "totals": []}
-
-        facts = storage.get_facts(self.start_date, self.end_date)
-
         for i in range((self.end_date - self.start_date).days  + 1):
             current_date = self.start_date + dt.timedelta(i)
             # date format in overview window fact listing
@@ -193,19 +240,18 @@ class StatsViewer:
                                                     fact_date,
                                                     "",
                                                     current_date.strftime('%Y-%m-%d'),
-                                                    ""])
+                                                    "", ""])
             by_day[self.start_date + dt.timedelta(i)] = {"duration": 0, "row_pointer": day_row}
 
+                
+                
+
         for fact in facts:
             start_date = fact["start_time"].date()
 
             duration = None
-            if fact["end_time"]: # not set if just started
-                delta = fact["end_time"] - fact["start_time"]
-                duration = 24 * delta.days + delta.seconds / 60
-            elif fact["start_time"].date() == dt.date.today():
-                delta = dt.datetime.now() - fact["start_time"]
-                duration = 24 * delta.days + delta.seconds / 60
+            if fact["delta"]:
+                duration = 24 * fact["delta"].days + fact["delta"].seconds / 60
 
             self.fact_store.append(by_day[start_date]["row_pointer"],
                                    [fact["id"],
@@ -213,75 +259,83 @@ class StatsViewer:
                                     fact["name"],
                                     stuff.format_duration(duration),
                                     fact["start_time"].strftime('%Y-%m-%d'),
-                                    fact["description"]
+                                    fact["description"],
+                                    fact["category"]
                                     ])
 
-            if fact["name"] not in by_activity: by_activity[fact["name"]] = 0
-            if fact["category"] not in by_category: by_category[fact["category"]] = 0
-
             if duration:
                 by_day[start_date]["duration"] += duration
-                by_activity[fact["name"]] += duration
-                by_category[fact["category"]] += duration
 
-        days = 30
-        if self.week_view.get_active():
-            days = 7
-
-
-        date_sort = lambda a, b: (b[4] < a[4]) - (a[4] < b[4])
-        totals["by_day"] = []
 
         for day in by_day:
             self.fact_store.set_value(by_day[day]["row_pointer"], 2,
                 stuff.format_duration(by_day[day]["duration"]))
-            if (self.end_date - self.start_date).days < 20:
-                strday = stuff.locale_to_utf8(day.strftime('%a'))
-                totals["by_day"].append([strday, by_day[day]["duration"] / 60.0, None, None, day])
-            else:
-                # date format in month chart in overview window (click on "month" to see it)
-                # prefix is "m_", letter after prefix is regular python format. you can use all of them
-                strday = _("%(m_b)s %(m_d)s") %  stuff.dateDict(day, "m_")
 
-                background = None
-                if day.weekday() in [5, 6]:
-                    background = 7
-
-                totals["by_day"].append([strday, by_day[day]["duration"] / 60.0, None, background, day])
-        totals["by_day"].sort(date_sort)
-            
-            
-        duration_sort = lambda a, b: (a[1] < b[1]) - (b[1] < a[1])
-        totals["by_activity"] = []
-        for activity in by_activity:
-            totals["by_activity"].append([activity, by_activity[activity] / 60.0])
-        totals["by_activity"].sort(duration_sort)
-        
-        #now we will limit bars to 6 and sum everything else into others
-        if len(totals["by_activity"]) > 12:
-            other_total = 0.0
-
-            for i in range(11, len(totals["by_activity"]) - 1):
-                other_total += totals["by_activity"][i][1]
-                
-            totals["by_activity"] = totals["by_activity"][:11]
-            totals["by_activity"].append([_("Other"), other_total, 1])
-        totals["by_activity"].sort(duration_sort) #sort again, since maybe others summed is bigger
-            
-        totals["by_category"] = []
-        for category in by_category:
-            totals["by_category"].append([category, by_category[category] / 60.0])
-        totals["by_category"].sort(duration_sort)
-        
 
         self.fact_tree.expand_all()
         
         self.get_widget("report_button").set_sensitive(len(facts) > 0)
 
 
-        week["totals"] = totals
+        
+    def get_totals(self, facts, all_days):
+        # get list of used activities in interval
+        activities = [act[0] for act in
+              storage.get_interval_activity_ids(self.start_date, self.end_date)]
+
+        # fill in the activity totals blanks
+        # don't want to add ability to be able to specify color per bar
+        # so we will be disguising our bar chart as multibar chart
+        activity_totals = {}
+        for act in activities:
+            activity_totals[act] = {}
+            for cat in self.popular_categories:
+                activity_totals[act][cat] = 0
+
+        # fill in the category totals blanks
+        day_category_totals = {}
+        for day in all_days:
+            day_category_totals[day] = {}
+            for cat in self.popular_categories:
+                day_category_totals[day][cat] = 0
             
-        return week
+        #now we do the counting
+        for fact in facts:
+            duration = None
+            start_date = fact['start_time'].date()
+            
+            if fact["end_time"]: # not set if just started
+                delta = fact["end_time"] - fact["start_time"]
+                duration = 24 * delta.days + delta.seconds / 60
+            elif start_date == dt.date.today():
+                delta = dt.datetime.now() - fact["start_time"]
+                duration = 24 * delta.days + delta.seconds / 60
+
+            activity_totals[fact['name']][fact['category']] += duration or 0
+            day_category_totals[start_date][fact['category']] += duration or 0
+
+
+        # convert dictionaries into lists so we don't have to care about keys anymore
+        res_categories = []
+        for day in all_days:
+            res_categories.append([day_category_totals[day][cat] / 60.0
+                                            for cat in self.popular_categories])
+            
+        #sort activities by duration, longest first
+        activity_totals = activity_totals.items()
+        activity_totals = sorted(activity_totals,
+                                 key = lambda(k,v): (max(v.values()), k),
+                                 reverse = True)
+        
+        activities = [] #we have changed the order
+        res_activities = []
+        for act in activity_totals:
+            activities.append(act[0])
+            res_activities.append([act[1][cat] / 60.0
+                                            for cat in self.popular_categories])
+
+        return {'keys': activities, 'values': res_activities}, \
+               {'keys': self.popular_categories, 'values': res_categories}
         
 
     def do_graph(self):
@@ -290,6 +344,7 @@ class StatsViewer:
         
         
         if self.start_date.year != self.end_date.year:
+        
             # overview label if start and end years don't match
             # letter after prefixes (start_, end_) is the one of
             # standard python date formatting ones- you can use all of them
@@ -305,13 +360,7 @@ class StatsViewer:
             # standard python date formatting ones- you can use all of them
             overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
 
-        if self.day_view.get_active():
-            # overview label for single day
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            overview_label = _("Overview for %(start_B)s %(start_d)s, %(start_Y)s") % dates_dict
-            dayview_caption = _("Day")
-        elif self.week_view.get_active():
+        if self.week_view.get_active():
             dayview_caption = _("Week")
         else:
             dayview_caption = _("Month")
@@ -321,28 +370,60 @@ class StatsViewer:
         label.set_text(overview_label)
 
         label2 = self.get_widget("dayview_caption")
-        label2.set_markup("<b>%s</b>" % (dayview_caption))
+        label2.set_markup("%s" % (dayview_caption))
         
-        facts = self.get_facts()
+        fact_list = storage.get_facts(self.start_date, self.end_date)
 
-        self.day_chart.plot(facts["totals"]["by_day"])
-        self.category_chart.plot(facts["totals"]["by_category"])
-        self.activity_chart.plot(facts["totals"]["by_activity"])
+        self.get_facts(fact_list)
+        
+        if not fact_list:
+            self.get_widget("graphs").hide()
+            self.get_widget("no_data_label").show()
+            return 
+        else:
+            self.get_widget("graphs").show()
+            self.get_widget("no_data_label").hide()
+            
+
+        all_days = [self.start_date + dt.timedelta(i)
+                    for i in range((self.end_date - self.start_date).days  + 1)]        
+
+        activity_totals, day_category_totals = self.get_totals(fact_list, all_days)
+
+        
+        
+        self.activity_chart.plot(activity_totals['keys'],
+                                  activity_totals['values'],
+                                  stack_keys = self.popular_categories)
 
 
+        #show days or dates depending on scale
+        if (self.end_date - self.start_date).days < 20:
+            day_keys = [day.strftime("%a") for day in all_days]
+        else:
+            day_keys = [_("%(m_b)s %(m_d)s") %  stuff.dateDict(day, "m_") for day in all_days]
 
 
+        self.day_chart.plot(day_keys, day_category_totals['values'],
+                             stack_keys = day_category_totals['keys'])
+
+        category_totals = [[sum(value) for value in zip(*day_category_totals['values'])]]
+        
+        category_keys = []
+        for i in range(len(day_category_totals['keys'])):
+            category_keys.append("%s %.1f" % (day_category_totals['keys'][i], category_totals[0][i]))
+        
+
+        self.category_chart.plot([_("Total")], category_totals,
+                                  stack_keys = category_keys)
+
 
     def get_widget(self, name):
         """ skip one variable (huh) """
-        return self.glade.get_widget(name)
+        return self._gui.get_object(name)
 
     def on_prev_clicked(self, button):
-        if self.day_view.get_active():
-            self.start_date -= dt.timedelta(1)
-            self.end_date -= dt.timedelta(1)
-        
-        elif self.week_view.get_active():
+        if self.week_view.get_active():
             self.start_date -= dt.timedelta(7)
             self.end_date -= dt.timedelta(7)
         
@@ -355,11 +436,7 @@ class StatsViewer:
         self.do_graph()
 
     def on_next_clicked(self, button):
-        if self.day_view.get_active():
-            self.start_date += dt.timedelta(1)
-            self.end_date += dt.timedelta(1)
-        
-        elif self.week_view.get_active():
+        if self.week_view.get_active():
             self.start_date += dt.timedelta(7)
             self.end_date += dt.timedelta(7)
         
@@ -373,11 +450,7 @@ class StatsViewer:
     
     def on_home_clicked(self, button):
         self.view_date = dt.date.today()
-        if self.day_view.get_active():
-            self.start_date = self.view_date
-            self.end_date = self.view_date
-        
-        elif self.week_view.get_active():
+        if self.week_view.get_active():
             self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
             self.start_date = self.start_date + dt.timedelta(self.locale_first_weekday())
             self.end_date = self.start_date + dt.timedelta(6)
@@ -419,7 +492,7 @@ class StatsViewer:
         if model[iter][0] == -1:
             return #not a fact
 
-        custom_fact = CustomFactController(None, model[iter][0])
+        custom_fact = CustomFactController(self, None, model[iter][0])
         custom_fact.show()
 
     def delete_selected(self):
@@ -461,7 +534,7 @@ class StatsViewer:
     def on_facts_row_activated(self, tree, path, column):
         selection = tree.get_selection()
         (model, iter) = selection.get_selected()
-        custom_fact = CustomFactController(None, model[iter][0])
+        custom_fact = CustomFactController(self, None, model[iter][0])
         custom_fact.show()
         
     def on_add_clicked(self, button):
@@ -475,7 +548,7 @@ class StatsViewer:
                                     int(selected_date[1]),
                                     int(selected_date[2]))
 
-        custom_fact = CustomFactController(selected_date)
+        custom_fact = CustomFactController(self, selected_date)
         custom_fact.show()
         
     def on_report_button_clicked(self, widget):
@@ -487,19 +560,28 @@ class StatsViewer:
         self.do_graph()
     
     def after_fact_update(self, event, date):
+        self.popular_categories = [cat[0] for cat in storage.get_popular_categories()]
         self.do_graph()
         
     def on_close(self, widget, event):
         dispatcher.del_handler('activity_updated', self.after_activity_update)
         dispatcher.del_handler('day_updated', self.after_fact_update)
-        return False
+        self.close_window()        
 
     def on_window_key_pressed(self, tree, event_key):
       if (event_key.keyval == gtk.keysyms.Escape
           or (event_key.keyval == gtk.keysyms.w 
               and event_key.state & gtk.gdk.CONTROL_MASK)):
-        self.window.destroy()
+        self.close_window()
     
+    
+    def close_window(self):
+        if not self.parent:
+            gtk.main_quit()
+        else:
+            self.window.destroy()
+            return False
+        
     def show(self):
-        self.window.show_all()
+        self.window.show()
 
