@@ -22,13 +22,22 @@
 # cells, columns, trees and other
 
 import gtk
-from hamster import storage
+from hamster import storage, SHARED_DATA_DIR
 import pango
 from pango import ELLIPSIZE_END
 
 import datetime as dt
 import locale
+import os
 
+def load_ui_file(name):
+    ui = gtk.Builder()
+    ui.add_from_file(os.path.join(SHARED_DATA_DIR, name))
+    return ui 
+
+def zero_hour(date):
+    return dt.datetime.combine(date.date(), dt.time(0,0))
+    
 # it seems that python or something has bug of sorts, that breaks stuff for
 # japanese locale, so we have this locale from and to ut8 magic in some places
 # see bug 562298
@@ -56,32 +65,69 @@ class CategoryCell(gtk.CellRendererText):
         self.set_property('scale', pango.SCALE_SMALL)
         self.set_property('yalign', 0.0)
 
-class ExpanderColumn(gtk.TreeViewColumn):
-    def __init__(self, label, text):
-        gtk.TreeViewColumn.__init__(self, label)
+
+insensitive_color = gtk.Label().style.fg[gtk.STATE_INSENSITIVE].to_string()
+def format_activity(name, category, description, pad_description = False):
+    "returns pango markup for activity with category and description"
+    text = name    
+    if category and category != _("Unsorted"):
+        text += """ - <span color="%s" size="x-small">%s</span>""" % (insensitive_color, category)
+
+    if description:
+        text+= "\n"
+        if pad_description:
+            text += "          "
+
+        text += """<span style="italic" size="small">%s</span>""" % description
         
+    return text
+    
+
+class ActivityColumn(gtk.TreeViewColumn):
+    def activity_painter(self, column, cell, model, iter):
+        activity_name = model.get_value(iter, self.name)
+        description = model.get_value(iter, self.description)
+        category = model.get_value(iter, self.category)
+        
+        markup = format_activity(activity_name, category, description)            
+        cell.set_property('markup', markup)
+        return
+        
+    def __init__(self, name, description, category = None):
+        gtk.TreeViewColumn.__init__(self)
+        
+        self.name, self.description, self.category = name, description, category
         self.set_expand(True)
         cell = gtk.CellRendererText()
-        cell.set_property('ellipsize', ELLIPSIZE_END)
         self.pack_start(cell, True)
-        self.set_attributes(cell, text=text)
+        cell.set_property("ellipsize", pango.ELLIPSIZE_END)
+        self.set_cell_data_func(cell, self.activity_painter)
 
-
-def format_duration(minutes):
-    if minutes == None:
-        return None
+def format_duration(minutes, human = True):
+    if not minutes:
+        if human:
+            return ""
+        else:
+            return "00:00"
     
     hours = minutes / 60
-    days = hours / 24
-    hours %= 24
     minutes = minutes % 60
     formatted_duration = ""
     
-    #TODO - convert to list comprehension or that other thing
-    if days > 0:
-        formatted_duration += "%d:" % days
-    formatted_duration += "%02d:%02d" % (hours, minutes)
-            
+    if human:
+        if minutes % 60 == 0:
+            # duration in round hours
+            formatted_duration += _("%dh") % (hours)
+        elif hours == 0:
+            # duration less than hour
+            formatted_duration += _("%dmin") % (minutes % 60.0)
+        else:
+            # x hours, y minutes
+            formatted_duration += _("%dh %dmin") % (hours, minutes % 60)
+    else:
+        formatted_duration += "%02d:%02d" % (hours, minutes)
+    
+    
     return formatted_duration
 
 def dateDict(date, prefix):
@@ -115,43 +161,11 @@ def dateDict(date, prefix):
 
     return res
 
-class DayStore(object):
-    """A day view contains a treeview for facts of the day and another
-       one for totals. It creates those widgets on init, use
-       fill_view(store) to fill the tree and calculate totals """
-
-    def __init__(self, date = None):
-        date = date or dt.date.today()
-        
-        # ID, Time, Name, Duration, Date, Description
-        self.fact_store = gtk.ListStore(int, str, str, str, str, str)
-        
-        self.facts = storage.get_facts(date)
-        self.totals = {}
-        
-        for fact in self.facts:
-            duration = None
-            
-            if fact["end_time"]: # not set if just started
-                delta = fact["end_time"] - fact["start_time"]
-                duration = 24 * delta.days + delta.seconds / 60
-            elif fact["start_time"].date() == dt.date.today():  # give duration to today's last activity
-                delta = dt.datetime.now() - fact["start_time"]
-                duration = 24 * delta.days + delta.seconds / 60
-            
-            fact_category = fact['category']
-            
-            if fact_category not in self.totals:
-                self.totals[fact_category] = 0
-
-            if duration:
-                self.totals[fact_category] += duration
-
-            current_duration = format_duration(duration)
-
-            self.fact_store.append([fact['id'], fact['name'], 
-                                    fact["start_time"].strftime("%H:%M"), 
-                                    current_duration,
-                                    fact["start_time"].strftime("%Y%m%d"),
-                                    fact["description"]])
-
+def escape_pango(text):
+    if not text:
+        return text
+    
+    text = text.replace ("&", "&amp;")
+    text = text.replace("<", "&lt;")
+    text = text.replace(">", "&gt;")
+    return text
